/* DIFFH - A diff frontend for output in HTML format.
 * Module:  GLOBAL (header)
 * Author:  Matt Giuca
 * Created: 5/2/06
 * Updated: 15/4/06
 *
 * Copyright (C) 2006 Matt Giuca <mattgiuca@users.sourceforge.net>
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either 
 * version 2 of the License, or (at your option) any later 
 * version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* Module: GLOBAL (header)
 * This module contains global declarations.
 */

/* GLOBAL INCLUDES */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* GLOBAL CONSTANTS */
#define FALSE   0
#define TRUE    1
#define DEFAULT_MALLOC 20       /* Start space for extendable mallocs */
#define DEFAULT_MALLOC_BIG 100

/* Parameter defaults */
#define DEFAULT_TABWIDTH 8
#define DEFAULT_CSS_BODYFONT        "sans-serif"
#define DEFAULT_CSS_BODYFONTSIZE    "x-small"
#define DEFAULT_CSS_BODYBACKCOLOR   "white"
#define DEFAULT_CSS_COLOR_UNCHANGED "#eee"
#define DEFAULT_CSS_COLOR_ADDED     "#cfc"
#define DEFAULT_CSS_COLOR_DELETED   "#ffa"
#define DEFAULT_CSS_COLOR_CHANGE    "red"

/* Program information */
#define PROGRAM_NAME    "Diffh"
#define COMMAND_NAME    "diffh"
#define AUTHORS         "Matt Giuca"
#define PROGRAM_VERSION "0.3.2"
#define PROGRAM_WEBSITE "http://diffh.sourceforge.net"
#define GPL_SHORT "This is free software.  You may redistribute copies of " \
    "it under the terms of\n" \
    "the GNU General Public License " \
    "<http://www.gnu.org/licenses/gpl.html>.\n" \
    "There is NO WARRANTY, to the extent permitted by law.\n"

/* HTML code constants */
/* All text written to the output is either here, or in
 * print_html_prologue/epilogue */
#define NL              "\n"

#define TABLE_START     "<table class=\"diff-table\" width=\"98%\" " \
    "cellpadding=\"0\" cellspacing=\"4px\">"
#define TABLE_END       "</table>"
#define TABLE_ROWSTART  "<tr>"
#define TABLE_ROWEND    "</tr>"
#define TABLE_CELLSTART_DIFFHEAD    "<td class=\"diff-heading\" " \
    "colspan=\"2\">"
#define TABLE_CELLSTART_HUNKHEAD    "<td class=\"diff-linenum\" " \
    "colspan=\"2\">"
#define TABLE_CELLSTART_DELETED     "<td class=\"diff-deleted\">"
#define TABLE_CELLSTART_ADDED       "<td class=\"diff-added\">"
#define TABLE_CELLSTART_UNCHANGED   "<td class=\"diff-unchanged\">"
#define TABLE_CELLEND   "</td>"
#define TABLE_CELL_DELETED          "<td>-</td>"
#define TABLE_CELL_ADDED            "<td>+</td>"
#define TABLE_CELL_UNCHANGED        "<td></td>"
#define TABLE_CELL_UNCHANGED_EMPTY  "<td>&nbsp;</td>"
#define TABLE_CELLS_MISSING         "<td colspan=\"2\"></td>"

#define PAGETITLE_FORMAT_ONE            "Diff - %s"
#define PAGETITLE_FORMAT_TWO            "Diff - %s - %s"
#define DIFFHEAD_FORMAT_NAME_TIME       "%s" HBR "%s"
#define DIFFHEAD_FORMAT_NAME_VER_TIME   "%s &ndash; %s" HBR "%s"
#define LINENUM_FORMAT  "Line %d:"
#define FOOTER_TEXT     "Content generated by <a href=\"" PROGRAM_WEBSITE \
    "\">" PROGRAM_NAME "</a>."

#define HIGHLIGHT_ON_DELETED        "<span class=\"diff-d\">"
#define HIGHLIGHT_ON_ADDED          "<span class=\"diff-a\">"
#define HIGHLIGHT_OFF   "</span>"

/* <head> and child indentation */
#define HINDH1          "  "
#define HINDH2          "    "
/* Style sheet indentation */
#define HINDS1          "        "
#define HINDS2          "          "
/* <body> and child indentation */
#define HINDB1          "  "
#define HINDB2          "    "
/* <tr> and <td> indentation */
#define HINDT1          "      "
#define HINDT2          "        "
#define HBR             "<br />"

#define HT_NBSP         "&nbsp;"
#define HT_AMP          "&amp;"
#define HT_QUOT         "&quot;"
#define HT_LT           "&lt;"
#define HT_GT           "&gt;"

/* Command-line arguments */
#define ARG_USAGE       "--help"
#define ARG_VERSION     "--version"

/* GLOBAL VARIABLES */
extern FILE* outstream;     /* The global output stream */

/* Parameters */
extern int tabwidth;

/* CSS style properties */
extern char* custom_css_filename;
extern char* css_bodyfont;
extern char* css_bodyfontsize;
extern char* css_bodybackcolor;
extern char* css_color_unchanged;
extern char* css_color_added;
extern char* css_color_deleted;
extern char* css_color_change;

/* GLOBAL DATA TYPES */
typedef enum {
    LINE_UNCHANGED,
    LINE_DELETED,
    LINE_ADDED
} LineType;

typedef struct {
    int from_line_start;    /* Starting lines from given range */
    int to_line_start;
    char** line;            /* Each line of text */
    char* line_type;        /* See LineType (compressed as char) */
    int numlines;
} Hunk;

typedef struct {
    char* from_filename;
    char* from_time;
    char* from_version;
    char* to_filename;
    char* to_time;
    char* to_version;
    Hunk** hunk;
    int numhunks;
} UDiff;
